<?php
# Copyright(C) 2004-2007 INL
# Written by Eric Leblond <regit@inl.fr>
#            Vincent Deffontaines <gryzor@inl.fr>
#            Jean Gillaux <jean@inl.fr>
#            Damien Boucard <damien.boucard AT inl.fr>
#
# $Id: elt.class.php 13914 2008-03-28 08:26:26Z haypo $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
#
#  This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

require_once('pear/Net/IPv4.php');
require_once('html.php');
require_once('function.php'); // modifiedTimestamp()

/**
 * Plural -> singular name for XML tag name. Examples:
 *   'periodicities' => 'periodicity'
 *   'localsin' => 'localin'
 *   'resources' => 'resource'
 */
function tagname_singular($tag)
{
         $tag = preg_replace('/ies$/','y', $tag);
         $tag = preg_replace('/s$/','', $tag);
         $tag = preg_replace('/^locals/','local', $tag);
         return $tag;
}

function xml_read_elements(&$object, $parent_node, $tag_name="elt", $class="elt")
{
    foreach($parent_node->get_elements_by_tagname($tag_name) as $node)
    {
        $item = new $class($node);
        $object->add_elt($item);
    }
}

class elt {
    var $datas;

    function elt($data, $data_type='xml', $id=0, $element_type=null)
    {
        if ($data_type == 'xml') {
            $struct=$data->attributes();
            $this->datas = Array();
            if ($element_type)
                $this->datas['type'] = $element_type;
            foreach($struct as $attr) {
                $this->datas[$attr->name()] = $attr->value();
            }
        }else{
            $data["ID"]=$id;
            if ($element_type and !array_key_exists('type', $data))
                $data['type'] = $element_type;
            $this->datas = $data;
        }

        if (!array_key_exists('modified', $this->datas)) {
            $this->datas['modified'] = modifiedTimestamp();
        }

        $errmsg = $this->check_consistency();
        if ($errmsg) {
            $errmsg = sprintf(_('%s consistency error: %s!'), $this->str(), $errmsg);
            throw new Exception($errmsg);
        }
    }

    function xmldump($xml,$curelt){
        $node=$xml->create_element( 'elt' );
        foreach($this->datas as $key=>$data){
          if (($data != '--') and ($data != '') and ($key!='invalid'))
                $node->set_attribute($key,$data);
        }
        $curelt->append_child($node);
    }
    function express($base=null) {
        $result=array();
        if (($this->datas['type'] == 'link')){
            $base->express_elt($this->datas['linkid'],'all',$base);
        } else {
            foreach($this->datas as $key=>$data){
                $result[$key]=$data;
            }
        }
        return($result);
    }

    function str()
    {
        $text = _('Element');
        if (array_key_exists('name', $this->datas))
            $text .= ' "'.$this->datas['name'].'"';
        $info = Array();
        if (array_key_exists('ID', $this->datas) and $this->datas['ID'])
            $info[] = '#'.$this->datas['ID'];
        if (array_key_exists('type', $this->datas))
            $info[] = 'type "'.$this->datas['type'].'"';
        if ($info)
            $text .= ' ('.implode(', ', $info).')';
        return $text;
    }

    function check_consistency()
    {
        $type = array_get($this->datas, 'type', '');
        if (!check_element_type($type)) {
            return sprintf(_('Invalid element type: "%s"'), $type);
        }

        // desc.xml elements have no identifier
        if (!in_array($type, Array('ipv4', 'connection', 'address', 'mark')) && !$this->datas['ID']) {
            return _("Invalid identifier");
        }

        if ($type == 'ipv4' or $type == 'address') {
            return $this->check_address_consistency();

        } else if ($type == 'periodicity') {
            return $this->check_periodicity_consistency();

        } else if ($type == 'nufw') {
            $group = array_get($this->datas, 'group', '');
            if (!check_nb($group, false))
                return sprintf(_('Invalid NuFW user group: "%s"'), $group);

        } else if ($type == 'proto') {
            return $this->check_protocol_consistency();
        }
        return '';
    }

    function check_periodicity_consistency()
    {
        # Check hour
        $starthour = (int)array_get($this->datas, 'starthour', '-1');
        $stophour= (int)array_get($this->datas, 'stophour', '-1');
        if ($starthour < 0 or 24 < $starthour) {
            return sprintf(_('Invalid start hour value (%s)'), $starthour);
        }
        if ($stophour < 0 or 24 < $stophour) {
            return sprintf(_('Invalid start hour value (%s)'), $starthour);
        }
        if ($stophour < $starthour) {
            return _('Stop hour is before start hour');
        }
        if ($stophour == $starthour) {
            return _('Period duration is nul: start hour = stop hour');
        }

        # Check days
        $startday = (int)array_get($this->datas, 'startday', '0');
        $stopday = (int)array_get($this->datas, 'stopday', '0');
        if ($startday < 1 or 7 < $startday) {
            return sprintf(_('Invalid start day value (%s)'), $startday);
        }
        if ($stopday < 1 or 7 < $stopday) {
            return sprintf(_('Invalid stop day value (%s)'), $stopday);
        }
        if ($stopday < $startday) {
            return _('Stop day is before start day');
        }
        return '';
    }

    function check_address_consistency()
    {
        // Get IPv4 address
        if (array_key_exists('net', $this->datas)) {
            // acls.xml <elt>
            $ip = $this->datas['net'];
        } else if (array_key_exists('addr', $this->datas)) {
            // desc.xml <net>
            $ip = $this->datas['addr'];
        } else {
            return _('Empty IPv4 address');
        }
        if (trim($ip) == '')
            return _('Empty IPv4 address');

        // Reject loopback address
        if (Net_IPv4::ipInNetwork($ip, '127.0.0.0/8'))
            return sprintf(_('Local IPv4 address ("%s") is forbidden'), $ip);

        return '';
    }

    function check_protocol_consistency()
    {
        // Check protocol
        $protocol = array_get($this->datas, 'proto', '');
        if (!$protocol || !check_protocol($protocol))
        {
            return sprintf(_('Invalid protocol: "%s"'), $protocol);
        }

        // Check source and destination port ranges
        if ($protocol == 'tcp' or $protocol == 'udp') {
            $syntax = _('a valid range is a port in 1..65535 (eg. "80) or a range of two ports (eg. "1024:65535")');
            $value = array_get($this->datas, 'sport', '');
            if ($value and !check_port_range($value))
            {
                return sprintf(_('Invalid port range for %s: "%s"; %s'),
                        'source port', $value, $syntax);
            }

            $value = array_get($this->datas, 'dport', '');
            if (!check_port_range($value))
            {
                return sprintf(_('Invalid port range for %s: "%s"; %s'),
                        'destination port', $value, $syntax);
            }
        } else {
            $sport = array_get($this->datas, 'sport');
            $dport = array_get($this->datas, 'dport');
            if ($sport or $dport) {
                return sprintf(_('The protocol "%s" has no source or destination port'), $protocol);
            }
        }

        // Check ICMP type
        if ($protocol == 'icmp') {
            $value = array_get($this->datas, 'icmptype', '');
            if ($value and !check_icmp_types($value)) {
                return sprintf(_('Invalid ICMP type: "%s"'), $value);
            }
        } else {
            $value = array_get($this->datas, 'icmptype');
            if ($value and $value != '--') {
                return sprintf(_('The protocol "%s" has no ICMP type'), $protocol);
            }
        }
        return '';
    }

    function get_id(){
      return $this->datas["ID"];
    }

    function get_obj()
    {
      $result=array();
      foreach ($this->datas as $key=>$obj)
      {
        $result{$key}=$obj;
      }
      return $result;
    }
}

class set
{
    var $elts;

    function add_elt($elt, $id=null)
    {
        if (is_null($id)) {
            $id = $elt->get_id();
        }
        $this->_check_new_element($elt, $id);
        if (array_key_exists($id, $this->elts)) {
            throw new Exception(sprintf(_('Duplicate identifier (%s).'), $id));
        }
        $this->elts[$id] = $elt;
    }

    function replace_elt($elt, $id=null)
    {
        if (is_null($id)) {
            $id = $elt->get_id();
        }
        if (array_key_exists($id, $this->elts)) {
            $old_elt = $this->elts[$id];
            unset ($this->elts[$id]);
        } else {
            $old_elt = null;
        }
        try {
            // Try to add the new element
            $this->add_elt($elt, $id);
        } catch (Exception $err) {
            // On error: restore previous element
            $this->elts[$id] = $old_elt;
            throw $err;
        }
    }

    function _check_new_element($elt, $id)
    {
        if (array_key_exists($id, $this->elts)) {
            throw new Exception(sprintf(_('Duplicate identifier (%s).'), $id));
        }
    }

    function has_elt($elt)
    {
        return array_key_exists($elt, $this->elts);
    }

    function get_elt($elt){
        if (array_key_exists($elt, $this->elts))
            return $this->elts[$elt];
        else
            return null;
    }

    function del_elt($index)
    {
        unset ($this->elts[$index]);
    }
}

class obj_set extends set
{
    function obj_set($name, $hash=array(), $check_dup_names=true)
    {
        $this->name = $name;
        $this->elts = Array();
        $this->check_dup_names = $check_dup_names;
        foreach ($hash as $id=>$elt)
        {
            $this->add_elt($elt, $id);
        }
    }

    function str()
    {
        return $this->name;
    }

    function get_max_order($from=-2,$to=-2){
      if (($from == -2) or ($to == -2))
      {
        return -1;
      }
      $max=0;
      if (isset($this->elts))
      {
        foreach (($this->elts) as $elts){
          $local = $elts->get_max_elt_order($from,$to);
          if ($local>$max)
            $max = $local;
        }
      }
      return($max);
    }

    function get_max_order_id(){
      $max=0;
      if (isset($this->elts))
      {
        foreach (($this->elts) as $elts){
          $local = $elts->get_max_elt_order_id();
          if ($local>$max)
            $max = $local;
        }
      }
      return($max);
    }

    function xmldump($xml,$curelt){
         $tagname = tagname_singular($curelt->tagname);
         if (isset($this->elts))
         foreach ($this->elts as $key=>$elt){
               $node = $xml->create_element($tagname);
               if(!isset($elt)){
                    continue;
               }
               $elt->xmldump($xml,$node);
               $curelt->append_child($node);
         }
    }

    function express_elt($elt,$scope="one"){
        $this->elts[$elt]->express($scope,$base=$this);
    }

    function _check_new_element($elt, $id)
    {
        parent::_check_new_element($elt, $id);

        // Name validation is disabled?
        if (!$this->check_dup_names)
            return;

        $use_datas = isset($elt->datas);
        if ($use_datas)
            $name = $elt->datas['name'];
        else
            $name = $elt->name;
        if (!$name) {
            throw new Exception(_('An empty name is forbidden!'));
        }

        $seen = Array();
        foreach ($this->elts as $iter)
        {
            if ($use_datas)
                $iter_name = $iter->datas['name'];
            else
                $iter_name = $iter->name;
            if ($iter_name == $name)
            {
                $msg = sprintf(
                    _('The name "%s" is used more than once for %s, please change the name of %s'),
                    $name, $this->str(), $elt->str());
                throw new Exception($msg);
            }
        }
    }

    function &get_elt_by_id($id){
        if (array_key_exists($id, $this->elts)) {
            return $this->elts[$id];
        } else {
            $res=null;
            return $res;
        }
    }
    function get_max_elt_id(){
      $max=0;
      if (!isset($this->elts))
        return(0);
      foreach (($this->elts) as $elts){
        $local = $elts->get_max_elt_id();
        if ($local>$max)
          $max = $local;
      }
      return $max;
    }
    function new_elt_id(){
      return $this->get_max_elt_id() + 1;
    }
    function get_max_id(){
      $max=0;
      if (isset($this->elts))
        foreach (($this->elts) as $elts){
          $local=$elts->get_id();
          if ($local>$max)
            $max=$local;
        }
      return $max;
    }
    function new_id(){
      return($this->get_max_id()+1);
    }
    function new_lower_id(){
      $max=$this->get_max_id();
      $seen=array();
      foreach (($this->elts) as $elts)
      {
        $seen[$elts->get_id()]=1;
      }
      for ($i=1; $i<=$max+1; $i++)
      {
        if (!array_key_exists($i, $seen))
          return $i;
      }
      return ($max+1);
    }

    function list_tab()
    {
        $result = Array();
        foreach ($this->elts as $elt) {
            $key = $elt->ID;
            $value = $elt->name;
            if (!$value)
                $value = $key;
            $result[$key] = $value;
        }
        return $result;
    }

    function ordered_list_tab()
    {
        $result = $this->list_tab();
        ksort($result);
        return $result;
    }

    function nb_obj(){
      $result=0;
      foreach ($this->elts as $elt){
        $result++;
      }
      return($result);
    }

    function reorder(){
        $i=1;
        $clone = new obj_set($this->name, array(), $this->check_dup_names);
        foreach($this->elts as $id_t =>$elt){
           $elt->ID = $i;
           $clone->add_elt($elt);
           $i++;
        }
        return $clone;
    }
}

class elt_set extends set {
    var $name;
    var $ID;

    function elt_set($ress, $type='xml', $id=0)
    {
        $this->elts = Array();
        if ($type != 'xml') {
            $this->ID=$id;
            if (array_key_exists('defaultaction', $ress))
            {
                $this->defaultaction=$ress['defaultaction'];
            }
            if (array_key_exists('defaultprefix', $ress))
            {
                $this->defaultprefix=$ress['defaultprefix'];
            }
            $this->name=$ress['name'];
            if (array_key_exists('comment',$ress))
                $this->comment=$ress['comment'];
            else
                $this->comment='';
        }else{
            $dfltac=$ress->get_attribute('defaultaction');
            if ((isset($dfltac)) and ($dfltac!='--') and ($dfltac!=''))
            {
                $this->defaultaction=$ress->get_attribute('defaultaction');
            }
            $dfltpre=$ress->get_attribute('defaultprefix');
            if (isset($dfltpre) and ($dfltpre!=''))
            {
                $this->defaultprefix=$ress->get_attribute('defaultprefix');
            }
            $this->name=$ress->get_attribute('name');
            $this->ID = $ress->get_attribute('ID');
            $this->comment = $ress->get_attribute('comment');
            if (!$this->ID) {
                $this->ID = $ress->get_attribute('id');
            }
        }

        if (!property_exists($this, 'modified')) {
            $this->modified = modifiedTimestamp();
        }

        $errmsg = $this->check_consistency();
        if ($errmsg)
        {
            $errmsg = sprintf(_('%s consistency error: %s!'), $this->str(), $errmsg);
            throw new Exception($errmsg);
        }
    }

    function xmldump($xml,$node)
    {
        $node->set_attribute('name',$this->name);
        if ((isset($this->defaultaction)) and ($this->defaultaction!='--') and ($this->defaultaction!=''))
        {
          $node->set_attribute('defaultaction',$this->defaultaction);
        }
        if (isset($this->defaultprefix) and ($this->defaultprefix!=''))
        {
          $node->set_attribute('defaultprefix',$this->defaultprefix);
        }
        $node->set_attribute('ID',$this->ID);
        $node->set_attribute('comment',$this->comment);
        if (isset($this->elts))
        {
          foreach ($this->elts as $elt){
                $elt->xmldump($xml,$node);
          }
        }
    }

    function str()
    {
        $text = _('Element set');
        if ($this->name)
            $text .= ' "'.$this->name.'"';
        if ($this->ID)
            $text .= ' (#'.$this->ID.')';
        return $text;
    }

    function check_consistency()
    {
        if (!$this->ID) {
            return 'Invalid identifier';
        }
        return '';
    }

    function modify_param($key,$value){
        if (!in_array($key, Array('name', 'comment', 'modified',
                    'defaultaction', 'defaultprefix')))
        {
            throw new Exception(sprintf(
                _('Unable to change value of the parameter "%s" of %s.'),
                $key, $this->str()));
        }
        $this->$key = $value;
    }

    function get_id(){
      return($this->ID);
    }

    function get_max_elt_id(){
        $max=0;
        foreach ($this->elts as $elt){
            $local=$elt->get_id();
            if ($local>$max)
              $max=$local;
        }
        return $max;
    }
    function list_tab()
    {
        $result = Array();
        foreach ($this->elts as $elt){
          if (array_key_exists('name', $elt->datas) and $elt->datas['name']!=''){
            $result{$elt->datas['ID']} = ($elt->datas['name']);
          }else{
            $result{$elt->datas['ID']} = $elt->datas['ID'];
          }
        }
      return $result;
    }
    function get_obj()
    {
      $result = Array();
      foreach ($this as $key=>$obj)
      {
        // Skip elts, that contains stuff from subelements
        if ($key=='elts')
          continue;
        $result{$key}=$obj;
      }
      return $result;
    }

    function read_elements($parent_node, $tag_name="elt")
    {
        xml_read_elements($this, $parent_node, $tag_name);
    }
}

function max_elt($array)
{
  $max=0;
  if (isset($array) and ($array!=''))
    foreach ($array as $elt)
    {
      $local=$elt->get_ID();
      if ($local>$max)
        $max=$local;
    }
  return($max);
}

function create_check_element_link($page, $obj, $elt=null, $use_url=true)
{
    if ($use_url) {
        $text = $obj->name;
        if ($elt) {
            $text .= " -> ".$elt->datas['name'];
        }

        $url = $page.'?elt='.$obj->ID;
        if ($elt) {
            $url .= '&sub_elt='.$elt->datas['ID'];
        }
        return createLink($url, $text);
    } else {
        $text = $obj->name.' (#'.$obj->ID.')';
        if ($elt) {
            $text .= " -> ".$elt->datas['name'].' (#'.$elt->datas['ID'].')';
        }
        return htmlspecialchars($text);
    }
}

?>
